/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.ogc.gml.v_3_1_1.dwr;

import net.opengis.gml.v_3_1_1.MultiPolygonType;
import net.opengis.gml.v_3_1_1.PolygonPropertyType;
import net.opengis.gml.v_3_1_1.PolygonType;

import org.directwebremoting.extend.MarshallException;
import org.jvnet.jaxb2_commons.locator.DefaultRootObjectLocator;
import org.jvnet.ogc.gml.v_3_1_1.jts.ConversionFailedException;

import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Polygon;

public class MultiPolygonTypeConverter
    extends
    AbstractGeometryTypeConverter<MultiPolygonType, double[][][][]> {

  private final PolygonTypeConverter polygonTypeConverter = new PolygonTypeConverter();

  @Override
  protected double[][][][] createCoordinates(MultiPolygonType multiPolygonType)
      throws MarshallException {

    try {
      final MultiPolygon multiPolygon = (MultiPolygon) getConverter().createGeometry(
          new DefaultRootObjectLocator(multiPolygonType),
          multiPolygonType);

      final int numGeometries = multiPolygon.getNumGeometries();
      double[][][][] coordinates = new double[numGeometries][][][];
      for (int index = 0; index < numGeometries; index++) {
        coordinates[index] = polygonTypeConverter.createCoordinates((Polygon) multiPolygon
            .getGeometryN(index));
      }
      return coordinates;
    }
    catch (ConversionFailedException cfex) {
      throw new MarshallException(MultiPolygonType.class, cfex);
    }
  }

  @Override
  protected String getGeometryType() {
    return "MultiPolygon";
  }

  @Override
  protected MultiPolygonType createGeometry(double[][][][] coordinates) {

    final MultiPolygonType multiPolygonType = new MultiPolygonType();

    for (int index = 0; index < coordinates.length; index++) {
      final double[][][] polygonCoordinates = coordinates[index];

      PolygonType polygonType = polygonTypeConverter.createGeometry(polygonCoordinates);
      PolygonPropertyType polygonPropertyType = new PolygonPropertyType();
      polygonPropertyType.setPolygon(polygonType);

      multiPolygonType.getPolygonMember().add(polygonPropertyType);

    }
    return multiPolygonType;

  }

  @Override
  protected Class<double[][][][]> getCoordinatesType() {
    return double[][][][].class;
  }
}
